#include <ole2.h>
#include "koala.h"

/*
 * CKoala::CKoala
 * CKoala::~CKoala
 *
 * Parameters (Constructor):
 *  pUnkOuter       LPUNKNOWN of a controlling unknown.
 *  pfnDestroy      PFNDESTROYED to call when an object
 *                  is destroyed.
 */

CKoala::CKoala(LPUNKNOWN pUnkOuter, PFNDESTROYED pfnDestroy)
    {
    m_pValue = new unsigned int;
    m_cRef=0;
    m_pUnkOuter=pUnkOuter;
    m_pfnDestroy=pfnDestroy;
    m_pITINeutral = NULL;
    m_pIUnkStdDisp = NULL;
    return;
    }

CKoala::~CKoala(void)
    {
      if (NULL != m_pIUnkStdDisp){
	m_pIUnkStdDisp->Release();
	m_pIUnkStdDisp = NULL;
      }
	
      if (NULL != m_pITINeutral){
	m_pITINeutral->Release();
	m_pITINeutral = NULL;
      }

    return;
    }



/*
 * CKoala::Init
 *
 * Purpose:
 *  Performs any intiailization of a CKoala that's prone to failure
 *  that we also use internally before exposing the object outside.
 *
 * Parameters:
 *  None
 *
 * Return Value:
 *  BOOL            TRUE if the function is successful,
 *                  FALSE otherwise.
 */

BOOL CKoala::Init(void)
    {
    HRESULT hr;
    ITypeLib* pITypeLib;
    LPUNKNOWN pIUnknown = this;

    //////////////////////////////////////////////////////////////////////
    // Create the Dispatch Interface
    
    hr = LoadRegTypeLib(LIBID_KoalaTlbLibrary, 1, 0, LANG_NEUTRAL, &pITypeLib);
    if (FAILED(hr)){
      return hr;
    }
    
    hr = pITypeLib->GetTypeInfoOfGuid(IID_IKoala, &m_pITINeutral);
    pITypeLib->Release();
    if (FAILED(hr)){
      return hr;
    }
    
    hr = CreateStdDispatch(pIUnknown, static_cast<IKoala*>(this),
			   m_pITINeutral, &m_pIUnkStdDisp);
    if (FAILED(hr)){
      m_pITINeutral->Release();
      return hr;
    }
    
    *m_pValue = 555;
    return TRUE;
    }




/*
 * CKoala::QueryInterface
 * CKoala::AddRef
 * CKoala::Release
 *
 * Purpose:
 *  IUnknown members for CKoala object.
 */

STDMETHODIMP CKoala::QueryInterface(REFIID riid, void** ppv)
    {
    *ppv=NULL;
    ULONG ref;

    if (IID_IUnknown==riid)
        *ppv=this;

    if (IID_IDispatch==riid || DIID_DIKoala==riid)
      return m_pIUnkStdDisp->QueryInterface(IID_IDispatch, ppv);


    if (NULL!=*ppv)
        {
        ref = ((LPUNKNOWN)*ppv)->AddRef();
        return S_OK;
        }

    return E_NOINTERFACE;
    }


STDMETHODIMP_(ULONG) CKoala::AddRef(void)
    {
    return ++m_cRef;
    }


STDMETHODIMP_(ULONG) CKoala::Release(void)
    {
    if (0L!=--m_cRef)
        return m_cRef;

    /*
     * Tell the housing that an object is going away so it can
     * shut down if appropriate.
     */
    if (NULL!=m_pfnDestroy)
        (*m_pfnDestroy)();

    delete this;
    return 0;
    }
